@@ -1,18 +1,28 @@
 /***************************************************************************
-                          filelist.cpp  -  description
-                             -------------------
-    begin                : Sun May 11 2003
-    copyright            : (C) 2003 by Elad Lahav
-    email                : elad_lahav@users.sf.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
+ *
+ * Copyright (C) 2005 Elad Lahav (elad_lahav@users.sourceforge.net)
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
  ***************************************************************************/
 
 #include <qlineedit.h>
@@ -28,14 +38,20 @@
  * @param	szName	The widget's name
  */
 FileList::FileList(QWidget* pParent, const char* szName) :
-	SearchList(1, pParent, szName)
+	SearchList(1, pParent, szName),
+	m_sRoot("/")
 {
 	// Set the list's columns
 	m_pList->addColumn("");
 	m_pList->addColumn(i18n("File"));
 	m_pList->addColumn(i18n("Path"));
-	// Sort only when asked by user
-	m_pList->setSortColumn(m_pList->columns() + 1);
+	
+	// Sort only when asked to by the user
+	if (Config().getAutoSortFiles())
+		m_pList->setSortColumn(1);
+	else
+		m_pList->setSortColumn(m_pList->columns() + 1);
+	
 	m_pList->setAllColumnsShowFocus(true);
 	
 	// Set colours and font
@@ -60,7 +76,7 @@ FileList::~FileList()
  */
 void FileList::addItem(const QString& sFilePath)
 {
-	QString sFileType, sFileName;
+	QString sFileType, sFileName, sPath;
 	int nTypePos;
 		
 	// Get the file's MIME-type (empty string for file names without an
@@ -69,9 +85,17 @@ void FileList::addItem(const QString& sFilePath)
 	if (nTypePos > -1)
 		sFileType = sFilePath.mid(nTypePos + 1);
 
-	// Extract the file name and create the list item
+	// Extract the file name
 	sFileName = sFilePath.mid(sFilePath.findRev('/') + 1);
-	new QListViewItem(m_pList, sFileType, sFileName, sFilePath);
+	
+	// If a root path has been set, use a $ sign instead of that part of the
+	// path
+	sPath = sFilePath;
+	if (m_sRoot != "/")
+		sPath.replace(m_sRoot, "$");
+	
+	// Create the list item
+	new QListViewItem(m_pList, sFileType, sFileName, sPath);
 }
 
 /**
@@ -101,8 +125,12 @@ void FileList::processItemSelected(QListViewItem* pItem)
 {
 	QString sPath;
 
-	// Get the file path and submit a request to open it for editing
+	// Get the file path (replace the root symbol, if required)
 	sPath = pItem->text(2);
+	if (sPath.startsWith("$"))
+		sPath.replace("$", m_sRoot);
+		
+	// Submit a request to open the file for editing
 	emit fileRequested(sPath, 0);
 }
 
@@ -113,12 +141,35 @@ void FileList::applyPrefs()
 {
 	// Apply colour settings
 	m_pList->setPaletteBackgroundColor(Config().getColor(
-		KScopeConfig::FileList, KScopeConfig::Background));
+		KScopeConfig::FileListBack));
 	m_pList->setPaletteForegroundColor(Config().getColor(
-		KScopeConfig::FileList, KScopeConfig::Foreground));
+		KScopeConfig::FileListFore));
 	m_pList->setFont(Config().getFont(KScopeConfig::FileList));
 }
 
+/**
+ * Associates a root directory with this list.
+ * For each file in the list, the part of the path corresponding to the root
+ * is replaced with a $ sign.
+ * @param	sRoot	The new root path
+ */
+void FileList::setRoot(const QString& sRoot)
+{
+	QListViewItem* pItem;
+	QString sPath;
+	
+	// Store the new root
+	m_sRoot = sRoot;
+	
+	// Update all items in the list
+	for (pItem = m_pList->firstChild(); pItem != NULL; 
+		pItem = pItem->nextSibling()) {
+		sPath = pItem->text(2);
+		sPath.replace(m_sRoot, "$");
+		pItem->setText(2, sPath);
+	}
+}
+
 /**
  * Sets the keyboad focus to the search box.
  * This slot is connected to a keyboard shortcuts for quick access to the
@@ -128,3 +179,5 @@ void FileList::slotSetFocus()
 {
 	m_pEdit->setFocus();
 }
+
+#include "filelist.moc"
