@@ -1,18 +1,28 @@
 /***************************************************************************
-                          kscope.cpp  -  description
-                             -------------------
-    begin                : Fri May  9 09:16:06 IDT 2003
-    copyright            : (C) 2003 by Elad Lahav
-    email                : elad_lahav@users.sf.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
+ *
+ * Copyright (C) 2005 Elad Lahav (elad_lahav@users.sourceforge.net)
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
  ***************************************************************************/
 
 #include <qfile.h>
@@ -50,6 +60,9 @@
 #include "cscopemsgdlg.h"
 #include "symboldlg.h"
 #include "symbolcompletion.h"
+#include "queryviewdlg.h"
+#include "graphwidget.h"
+#include "welcomedlg.h"
 
 /**
  * Class constructor.
@@ -75,14 +88,12 @@ KScope::KScope(QWidget* pParent, const char* szName) :
 	m_pFileView = new FileView(this);
 	m_pFileList = m_pFileView->getFileList();
 	m_pMsgDlg = new CscopeMsgDlg(this);
+	m_pQueryDock = createDockWidget("Query Window", QPixmap());
+	m_pFileViewDock = createDockWidget("File List Window", QPixmap());
 
 	// Connect menu and toolbar items with the object's slots
 	initActions();
 	
-	// Create the initial GUI (no active part)
-	setXMLFile("kscopeui.rc");
-	createShellGUI();
-
 	// Create all child widgets
 	initMainWindow();
 
@@ -145,16 +156,24 @@ KScope::KScope(QWidget* pParent, const char* szName) :
 		
 	// Store main window settings when closed
 	setAutoSaveSettings();
+	
+	// Initialise arrow head drawing
+	GraphWidget::setArrowInfo(20, 15);
+	
+	// Use a maximised window the first time
+	if (Config().isFirstTime())
+		showMaximized();
+
+	// Show the Welcome message
+	if (Config().showWelcomeDlg()) {
+		show();
+		slotShowWelcome();
+	}
 
 	// If this is the first time the user has launched KScope, prompt him/her
 	// to configure the global parameters
-	if (Config().isFirstTime()) {
-		showMaximized(); // Use a maximised window the first time
-		KMessageBox::information(0, i18n("Welcome to KScope!\nIt appears to "
-			"be the first time you are running this programme.\nPlease fill "
-			"in the required configuration parameters."));
-		slotConfigure();
-	}
+	if (Config().isFirstTime())
+		slotConfigure();		
 }
 
 /**
@@ -178,15 +197,20 @@ KScope::~KScope()
  */
 void KScope::initActions()
 {
+	KToggleAction* pLockAction;
+	
 	// File menu
 	KStdAction::openNew(this, SLOT(slotNewFile()), actionCollection());
 	KStdAction::open(this, SLOT(slotOpenFile()), actionCollection());
 	KStdAction::close(this, SLOT(slotCloseEditor()), actionCollection());
 	KStdAction::quit(this, SLOT(slotClose()), actionCollection());
 
-	(void)new KAction("Go to File List", KShortcut("Ctrl+Shift+O"),
+	(void)new KAction(i18n("Go to File List"), KShortcut("Ctrl+Shift+O"),
 		m_pFileList, SLOT(slotSetFocus()), actionCollection(),
 		"file_open_file_from_list");
+	(void)new KAction(i18n("Save Al&l"), 
+		"save_all",	KShortcut("Ctrl+L"), m_pEditTabs, SLOT(slotSaveAll()),
+		actionCollection(), "file_save_all");
 	
 	// Edit menu
 	m_pExtEditAction = new KAction(i18n("Edit in E&xternal Editor"),
@@ -232,24 +256,24 @@ void KScope::initActions()
 		"cscope_including");
 	(void)new KAction(i18n("&Quick Definition"), 0, KShortcut("Ctrl+]"), this,
 		SLOT(slotQueryQuickDef()), actionCollection(), "cscope_quick_def");
-	(void)new KAction(i18n("Ca&ll Tree..."), 0, KShortcut("Ctrl+\\"), this,
+	(void)new KAction(i18n("Call &Graph..."), 0, KShortcut("Ctrl+\\"), this,
 		SLOT(slotCallTree()), actionCollection(), "cscope_call_tree");
-	(void)new KAction(i18n("Re&build database"),
-		Pixmaps().getPixmap(KScopePixmaps::ToolbarReload), 0, this,
+	(void)new KAction(i18n("Re&build database"), "reload", 0, this,
 		SLOT(slotRebuildDB()), actionCollection(),
 		"cscope_rebuild");
-	(void)new KAction(i18n("Close &Query"), 0,
-		0, this, SLOT(slotCloseQuery()), actionCollection(),
-		"cscope_close_query");
 
 	// Go menu
-	(void)new KAction(i18n("&Previous Position"),
-		Pixmaps().getPixmap(KScopePixmaps::GoBack), 
+	(void)new KAction(i18n("P&revious Result"), "up", 
+		KShortcut(QKeySequence(ALT + Key_Up)), m_pQueryWidget,
+		SLOT(slotPrevResult()), actionCollection(), "go_prev_result");
+	(void)new KAction(i18n("N&ext Result"), "down", 
+		KShortcut(QKeySequence(ALT + Key_Down)), m_pQueryWidget,
+		SLOT(slotNextResult()), actionCollection(),
+		"go_next_result");
+	(void)new KAction(i18n("&Previous Position"), "previous", 
 		KShortcut(QKeySequence(ALT + Key_Left)), m_pQueryWidget,
-		SLOT(slotHistoryPrev()), actionCollection(),
-		"go_prev_pos");
-	(void)new KAction(i18n("&Next Position"),
-		Pixmaps().getPixmap(KScopePixmaps::GoForward),
+		SLOT(slotHistoryPrev()), actionCollection(), "go_prev_pos");
+	(void)new KAction(i18n("&Next Position"), "next", 
 		KShortcut(QKeySequence(ALT + Key_Right)), m_pQueryWidget,
 		SLOT(slotHistoryNext()), actionCollection(),
 		"go_next_pos");
@@ -258,31 +282,66 @@ void KScope::initActions()
 		"go_history");
 
 	// View menu
-	(void)new KAction(i18n("Show/Hide File List"), KShortcut("Ctrl+/"), this,
-		SLOT(slotToggleFileListDock()), actionCollection(),
+	m_pToggleFileViewAction = new KToggleAction(i18n("Show/Hide File List"),
+		"view_sidetree", KShortcut("Ctrl+/"), m_pFileViewDock,
+		SLOT(changeHideShowState()), actionCollection(),
 		"view_toggle_filelist_dock");
-	(void)new KAction(i18n("Show/Hide Query Window"), KShortcut("Ctrl+."),
-		this, SLOT(slotToggleQueryDock()), actionCollection(),
-		"view_toggle_query_dock");
-	(void)new KAction(i18n("Show/Hide Query Buttons"), 0, this,
-		SLOT(slotToggleQueryButtons()), actionCollection(),
-		"view_toggle_query_buttons");
-	(void)new KAction(i18n("Show/Hide Tag List"), KShortcut("Ctrl+'"),
-		m_pEditTabs, SLOT(slotToggleTagList()), actionCollection(),
+	m_pToggleQueryWindowAction = new KToggleAction(
+		i18n("Show/Hide Query Window"), "view_top_bottom",
+		KShortcut("Ctrl+."), m_pQueryDock, SLOT(changeHideShowState()),
+		actionCollection(), "view_toggle_query_dock");
+	m_pToggleTagListAction = new KToggleAction(i18n("Show/Hide Tag List"),
+		"view_detailed", KShortcut("Ctrl+'"), m_pEditTabs,
+		SLOT(slotToggleTagList()), actionCollection(),
 		"view_toggle_tag_list");
 	
 	// Window menu
-	(void)new KAction(i18n("Close &All Windows..."), 0, 0, this,
+	(void)new KAction(i18n("Close &All"), 0, 0, this,
 		SLOT(slotCloseAllWindows()), actionCollection(), "window_close_all");
+	(void)new KAction(i18n("Go &Left"), 0, 
+		KShortcut(QKeySequence(ALT + SHIFT + Key_Left)),
+		m_pEditTabs, SLOT(slotGoLeft()), actionCollection(),
+		"window_go_left");
+	(void)new KAction(i18n("Go &Right"), 0,
+		KShortcut(QKeySequence(ALT + SHIFT + Key_Right)),
+		m_pEditTabs, SLOT(slotGoRight()), actionCollection(),
+		"window_go_right");
 		
 	// Settings menu
 	KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection());
 	KStdAction::keyBindings(this, SLOT(slotShortcuts()), actionCollection());
 	
+	// Help menu
+	(void)new KAction(i18n("Show &Welcome Message..."), 0, 0, this,
+		SLOT(slotShowWelcome()), actionCollection(), "help_welcome");
+	
+	// Query widget popup menu
+	(void)new KAction(i18n("&New"), "filenew", 0, m_pQueryWidget,
+		SLOT(slotNewQueryPage()), actionCollection(), "query_new");
+	(void)new KAction(i18n("&Refresh"), "reload", 0, m_pQueryWidget,
+		SLOT(slotRefreshCurrent()), actionCollection(), "query_refresh");
+	pLockAction = new KToggleAction(i18n("&Lock/Unlock"), "encrypted", 0,
+		m_pQueryWidget, SLOT(slotLockCurrent()), actionCollection(),
+		"query_toggle_locked");
+	(void)new KAction(i18n("&Close"), "fileclose", 0, m_pQueryWidget,
+		SLOT(slotCloseCurrent()), actionCollection(), "query_close");
+	
 	m_pExtEditAction->setEnabled(Config().useExtEditor());
 	
 	// Show a toolbar show/hide menu
 	setStandardToolBarMenuEnabled(true);
+	
+	// Create the initial GUI (no active part)
+	setXMLFile("kscopeui.rc");
+	createShellGUI();
+
+	// Associate the "Window" menu with the editor tabs widdget
+	m_pEditTabs->setWindowMenu((QPopupMenu*)factory()->container("window",
+		this));
+
+	// Associate the "Query" popup menu with the query widget
+	m_pQueryWidget->setPageMenu((QPopupMenu*)factory()->container(
+		"query_popup", this), pLockAction);
 }
 
 /**
@@ -306,23 +365,30 @@ void KScope::initMainWindow()
 	setView(pMainDock);
 	pMainDock->setEnableDocking(KDockWidget::DockNone);
 
-	// Associate the "Window" menu with the editor tabs widdget
-	m_pEditTabs->setWindowMenu((QPopupMenu*)factory()->container("window",
-		this));
-	
 	// Create the query window dock
-	m_pQueryDock = createDockWidget("Query Window", QPixmap());
 	m_pQueryDock->setWidget(m_pQueryWidget);
 	m_pQueryDock->manualDock(pMainDock, KDockWidget::DockBottom, 65);
 
+	// Update the relevant shell action when the dock is hidden through its
+	// close button
+	connect(m_pQueryDock, SIGNAL(headerCloseButtonClicked()), this,
+		SLOT(slotQueryDockClosed()));
+		
 	// Create the file view dock
-	m_pFileViewDock = createDockWidget("File List Window", QPixmap());
 	m_pFileViewDock->setWidget(m_pFileView);
 	m_pFileViewDock->manualDock(pMainDock, KDockWidget::DockRight, 80);
 	
+	// Update the relevant shell action when the dock is hidden through its
+	// close button
+	connect(m_pFileViewDock, SIGNAL(headerCloseButtonClicked()), this,
+		SLOT(slotFileViewDockClosed()));
+	
 	// Restore dock configuration
 	Config().loadWorkspace(this);
 	m_bHideQueryOnSelection = m_pQueryDock->isHidden();
+	m_pToggleFileViewAction->setChecked(m_pFileViewDock->isShown());
+	m_pToggleQueryWindowAction->setChecked(m_pQueryDock->isShown());
+	m_pToggleTagListAction->setChecked(Config().getShowTagList());
 }
 
 /**
@@ -363,9 +429,16 @@ void KScope::slotCreateProject()
 
 	// Prompt the user to close any active projects
 	if (m_pProjMgr->isOpen()) {
-		KMessageBox::error(0, i18n("Please close the active project before"
-			" creating a new one"));
-		return;
+		if (KMessageBox::questionYesNo(0, 
+			i18n("The current project needs to be closed before a new one is"
+			" created.\nWould you like to close it now?")) != 
+			KMessageBox::Yes) {
+			return;
+		}
+		
+		// Try to close the project.
+		if (!slotCloseProject())
+			return;
 	}
 	
 	// Display the "New Project" dialog
@@ -392,7 +465,7 @@ void KScope::slotOpenProject()
 	if (dlg.exec() == QDialog::Rejected)
 		return;
 
-	openProject(dlg.m_pPathEdit->text());
+	openProject(dlg.getPath());
 }
 
 /**
@@ -561,7 +634,24 @@ void KScope::slotQueryIncluding()
  */
 void KScope::slotQueryQuickDef()
 {
-	slotQuery(SymbolDlg::Definition, false);
+	QString sSymbol;
+	QueryViewDlg* pDlg;
+	uint nType;
+	
+	// Get the requested symbol and query type
+	nType = SymbolDlg::Definition;
+	if (!getSymbol(nType, sSymbol, false))
+		return;
+		
+	// Create a modeless query view dialogue
+	pDlg = new QueryViewDlg(QueryViewDlg::DestroyOnSelect, this);
+	
+	// Display a line when it is selected in the dialogue
+	connect(pDlg, SIGNAL(lineRequested(const QString&, uint)), this,
+		SLOT(slotShowEditor(const QString&, uint)));
+		
+	// Start the query
+	pDlg->query(nType, sSymbol);
 }
 
 /**
@@ -580,24 +670,15 @@ void KScope::slotCallTree()
 void KScope::slotRebuildDB()
 {
 	if (!m_pProjMgr->dbExists()) {
-		m_pProgressDlg = new KProgressDialog(this, 0, i18n("KScope"), 
-			i18n("Please wait while the database is built for the first "
-				"time"), true);
+		m_pProgressDlg = new ProgressDlg(i18n("KScope"), i18n("Please wait "
+			"while the database is built for the first time"), this);
 		m_pProgressDlg->setAllowCancel(false);
+		m_pProgressDlg->setValue(0);
 	}
 	
 	m_pCscopeBuild->rebuild();
 }
 
-/**
- * Handles the "Cscope->Close Query" command.
- * Closes the active query list.
- */
-void KScope::slotCloseQuery()
-{
-	m_pQueryWidget->slotCloseCurrent();
-}
-
 /**
  * Handles the "Settings->Configure Shortcuts..." command.
  * Displays the prferences dialog, which allows the user 
@@ -636,8 +717,10 @@ void KScope::slotProjectFilesChanged()
 	QStringList slArgs;
 	
 	// Refresh the file list
+	m_pFileList->setUpdatesEnabled(false);
 	m_pFileList->clear();
 	m_pProjMgr->fillList(m_pFileList);
+	m_pFileList->setUpdatesEnabled(true);
 	
 	// Rebuild the symbol database
 	if (isAutoRebuildEnabled())
@@ -681,9 +764,9 @@ void KScope::slotQuery(uint nType, bool bPrompt)
 		
 	if (nType == SymbolDlg::CallTree) {
 		// Create and display a call tree dialogue
-		pCallTreeDlg = m_pCallTreeMgr->addDialog(sSymbol);
-		if (pCallTreeDlg)
-			pCallTreeDlg->show();
+		pCallTreeDlg = m_pCallTreeMgr->addDialog();
+		pCallTreeDlg->setRoot(sSymbol);
+		pCallTreeDlg->show();
 	}
 	else {
 		// Run the requested query
@@ -691,7 +774,7 @@ void KScope::slotQuery(uint nType, bool bPrompt)
 		m_pQueryWidget->initQuery(nType, sSymbol);
 		
 		// Ensure Query Window is visible
-		slotShowQueryWindow();	
+		toggleQueryWindow(true);	
 	}
 }
 
@@ -720,7 +803,7 @@ void KScope::openProject(const QString& sDir)
 	setCaption(m_pProjMgr->getName());
 
 	// Set the root of the file tree
-	m_pFileView->setTreeRoot(m_pProjMgr->getRoot());
+	m_pFileView->setRoot(m_pProjMgr->getRoot());
 	
 	// Initialise Cscope and create a builder object
 	initCscope();
@@ -731,7 +814,7 @@ void KScope::openProject(const QString& sDir)
 		opt.nACDelay, opt.nACMaxEntries);
 	
 	// Create an initial query page
-	m_pQueryWidget->slotAddQueryPage();
+	m_pQueryWidget->addQueryPage();
 	
 	// If this is a new project (i.e., no source files are yet included), 
 	// display the project files dialogue
@@ -741,7 +824,9 @@ void KScope::openProject(const QString& sDir)
 	}
 	
 	// Fill the file list with all files in the project. 
+	m_pFileList->setUpdatesEnabled(false);
 	m_pProjMgr->fillList(m_pFileList);
+	m_pFileList->setUpdatesEnabled(true);
 	
 	// Reload previously-open files
 	loadOpenFiles();
@@ -766,31 +851,59 @@ void KScope::openProject(const QString& sDir)
  */
 void KScope::loadOpenFiles()
 {
-	QStringList slOpenFiles;
-	QStringList::Iterator itr;
-	int nSize, i;
+	OpenFileList list;
+	FileLocation* pLoc;
+	EditorPage* pPage;
 	
 	// Get the list of open files
-	m_pProjMgr->getOpenFiles(slOpenFiles);
-	nSize = slOpenFiles.size();
-	if (nSize == 0)
+	list.setAutoDelete(true);
+	m_pProjMgr->getOpenFiles(list);
+	if (list.isEmpty())
 		return;
 	
 	// Do not update the GUI when loading the editor parts of the initially
 	// hidden windows
 	m_bUpdateGUI = false;
 	
-	for (i = 0, itr = slOpenFiles.begin(); i < nSize - 1; i++, ++itr) {
-		if (QFile::exists(*itr))
-			addEditor(*itr);
+	for (pLoc = list.first(); pLoc != NULL; pLoc = list.next()) {
+		if (QFile::exists(pLoc->m_sPath)) {
+			pPage = addEditor(pLoc->m_sPath);
+			pPage->setCursorPos(pLoc->m_nLine, pLoc->m_nCol);
+		}
 	}
 	
-	// Megre the GUI of the visible editor part
+	// Merge the GUI of the visible editor part
 	m_bUpdateGUI = true;
 
-	// Load the last (visible) file 
-	if (QFile::exists(*itr))
-		addEditor(*itr);
+	// Set the active editor (or choose a default one)
+	if (m_pEditTabs->findEditorPage(m_pProjMgr->getLastOpenFile(), true) ==
+		NULL) {
+		m_pEditTabs->findEditorPage(list.last()->m_sPath, true);
+	}
+}
+
+/**
+ * Shows or hides the query dock window.
+ * This function is only called internally, not as a result of a user's
+ * workspace action (e.g., clicking the "Show/Hide Query Window" toolbar
+ * button). Therefore it does not reflect the user's preference, which is
+ * kept through the m_bHideQueryOnSelection variable.
+ * @param	bShow	true to show the window, false to hide it
+ */
+void KScope::toggleQueryWindow(bool bShow)
+{
+	// Remember the user's preferences
+	if (bShow)
+		m_bHideQueryOnSelection = m_pQueryDock->isHidden();
+	else
+		m_bHideQueryOnSelection = false;
+	
+	// Change the visibility state of the widget, if required
+	if (m_pQueryDock->isShown() != bShow)
+		m_pQueryDock->changeHideShowState();
+		
+	// Make sure the action's state matches the widget's
+	m_pToggleQueryWindowAction->setChecked(bShow);
 }
 
 /**
@@ -816,7 +929,7 @@ bool KScope::openCscopeOut(const QString& sFilePath)
 	initCscope();
 	
 	// Create an initial query page
-	m_pQueryWidget->slotAddQueryPage();
+	m_pQueryWidget->addQueryPage();
 	return true;
 }
 
@@ -884,36 +997,14 @@ void KScope::initCscope()
 		SLOT(slotBuildProgress(int, int)));
 	connect(m_pCscopeBuild, SIGNAL(finished(uint)), this,
 		SLOT(slotBuildFinished(uint)));
+	connect(m_pCscopeBuild, SIGNAL(aborted()), this,
+		SLOT(slotBuildAborted()));
 
 	// Show errors in a modeless dialogue
 	connect(m_pCscopeBuild, SIGNAL(error(const QString&)), this,
 		SLOT(slotCscopeError(const QString&)));
 }
 
-/**
- * Shows/Hides the filelist dock.
- */
-void KScope::slotToggleFileListDock()
-{
-	m_pFileViewDock->changeHideShowState();
-}
-
-/**
- * Shows/Hides the query dock.
- */
-void KScope::slotToggleQueryDock()
-{
-	m_pQueryDock->changeHideShowState();
-}
-
-/**
- * Shows/Hides the query buttons.
- */
-void KScope::slotToggleQueryButtons()
-{
-	m_pQueryWidget->showHideButtons();
-}
-
 /**
  * Closes the active project.
  * Closing a project involves closing all of the editor windows (prompting
@@ -922,7 +1013,8 @@ void KScope::slotToggleQueryButtons()
  */
 bool KScope::slotCloseProject()
 {
-	QStringList slOpenFiles;
+	QString sLastOpenFile;
+	OpenFileList lstOpenFiles;
 	QStringList slQueryFiles;
 	QStringList slCallTreeFiles;
 	
@@ -931,10 +1023,13 @@ bool KScope::slotCloseProject()
 		return true;
 	
 	// Close all open editor pages
-	m_pEditTabs->getOpenFiles(slOpenFiles);
-	if (!m_pEditTabs->removeAllPages())
-		return false;
-
+	if (m_pEditTabs->count() > 0) {
+		sLastOpenFile = m_pEditTabs->getCurrentPage()->getFilePath();
+		m_pEditTabs->getOpenFiles(lstOpenFiles);
+		if (!m_pEditTabs->removeAllPages())
+			return false;
+	}
+	
 	// Store locked queries, and close all query pages
 	m_pQueryWidget->saveLockedQueries(m_pProjMgr->getPath(), slQueryFiles);
 	m_pQueryWidget->slotCloseAll();
@@ -944,7 +1039,9 @@ bool KScope::slotCloseProject()
 	m_pCallTreeMgr->closeAll();
 	
 	// Save current session
-	m_pProjMgr->setOpenFiles(slOpenFiles);
+	lstOpenFiles.setAutoDelete(true);
+	m_pProjMgr->setLastOpenFile(sLastOpenFile);
+	m_pProjMgr->setOpenFiles(lstOpenFiles);
 	m_pProjMgr->setQueryFiles(slQueryFiles);
 	m_pProjMgr->setCallTreeFiles(slCallTreeFiles);
 	
@@ -995,6 +1092,16 @@ void KScope::slotCompleteSymbol()
 		pPage->slotCompleteSymbol();
 }
 
+/**
+ * Handles the "Help->Show Welcome Message..." menu command.
+ * Displays the "Welcome" dialogue.
+ */
+void KScope::slotShowWelcome()
+{
+	WelcomeDlg dlg;
+	dlg.exec();
+}
+
 /**
  * Prompts the user for a symbol to query.
  * Shows a dialog with a line edit widget, where the user can enter a symbol
@@ -1042,7 +1149,7 @@ bool KScope::getSymbol(uint& nType, QString& sSymbol, bool bPrompt)
  * Opens a file in a new editor tab.
  * If an editor page already exists for the requested file, it is selected.
  * Otherwise, a new page is created, and the requested file is loaded.
- * @param	sFileName	The path of the file to open
+ * @param	sFilePath	The path of the file to open
  * @return	A pointer to the found or newly created editor page
  */
 EditorPage* KScope::addEditor(const QString& sFilePath)
@@ -1207,10 +1314,8 @@ void KScope::slotShowEditor(const QString& sFilePath, uint nLine)
 void KScope::slotQueryShowEditor(const QString& sFilePath, uint nLine)
 {
 	// Hide the query window, if it was hidden before a query was initiated
-	if (m_bHideQueryOnSelection && !m_pQueryDock->isHidden()) {
-		m_pQueryDock->changeHideShowState();
-		m_bHideQueryOnSelection = false;
-	}
+	if (m_bHideQueryOnSelection)
+		toggleQueryWindow(false);
 	
 	// Open an editor at the requested line
 	slotShowEditor(sFilePath, nLine);
@@ -1223,7 +1328,7 @@ void KScope::slotQueryShowEditor(const QString& sFilePath, uint nLine)
  */
 void KScope::slotHistoryShow()
 {
-	slotShowQueryWindow();
+	toggleQueryWindow(true);
 	m_pQueryWidget->selectActiveHistory();
 }
 
@@ -1248,11 +1353,17 @@ void KScope::slotNewFile()
  */
 void KScope::slotOpenFile()
 {
-	QString sFileName;
+	QStringList slFiles;
+	QStringList::Iterator itr;
 	
-	sFileName = KFileDialog::getOpenFileName();
-	if (!sFileName.isEmpty())
-		slotShowEditor(sFileName, 0);
+	// Prompt the user for the file(s) to open.
+	slFiles = KFileDialog::getOpenFileNames();
+	
+	// Open all selected files.
+	for (itr = slFiles.begin(); itr != slFiles.end(); ++itr) {
+		if (!(*itr).isEmpty())
+			slotShowEditor(*itr, 0);
+	}
 }
 
 /**
@@ -1301,7 +1412,7 @@ void KScope::slotBuildProgress(int nFiles, int nTotal)
 	
 	// Use the progress dialogue, if it exists (first time builds)
 	if (m_pProgressDlg) {
-		m_pProgressDlg->progressBar()->setValue((nFiles * 100) / nTotal);
+		m_pProgressDlg->setValue((nFiles * 100) / nTotal);
 		return;
 	}
 	
@@ -1330,6 +1441,32 @@ void KScope::slotBuildFinished(uint)
 		3000);
 }
 
+/**
+ * Called if the build process failed to complete.
+ * This slot is connected to the aborted() signal emitted by the builder
+ * process.
+ */
+void KScope::slotBuildAborted()
+{
+	// Delete the progress dialogue, if it exists (first time builds)
+	if (m_pProgressDlg) {
+		delete m_pProgressDlg;
+		m_pProgressDlg = NULL;
+	
+		// Display a failure message
+		KMessageBox::error(0, i18n("The database could not be built.\n"
+			"Cross-reference information will not be available for this "
+			"project.\n"
+			"Please ensure that the Cscope parameters were correctly "
+			"entered in the \"Settings\" dialogue."));		
+		return;
+	}
+	
+	// Show a message in the status bar
+	statusBar()->message(i18n("Building cross reference database...Failed"),
+		3000);	
+}
+
 /**
  * Applies the selected user preferences once the "Apply" or "OK" buttons in
  * the preferences dialog is clicked.
@@ -1345,19 +1482,6 @@ void KScope::slotApplyPref()
 	m_pExtEditAction->setEnabled(Config().useExtEditor());
 }
 
-/**
- * Displays the query window once a query has completed (in case it is not
- * already visible.)
- * If the window is not visible, this information is saved and the query
- * window is hidden again when the user makes a record selection.
- */
-void KScope::slotShowQueryWindow()
-{
-	// Show the query window, if currently hidden
-	m_bHideQueryOnSelection = m_pQueryDock->isHidden();
-	m_pQueryDock->makeDockVisible();
-}
-
 /**
  * Displays the current cursor position, whenever it is moved by the user.
  * This slot is connected to the cursorPosChanged() signal emitted by an
@@ -1451,3 +1575,27 @@ void KScope::slotDropEvent(QDropEvent* pEvent)
 	for (itr = list.begin(); itr != list.end(); ++itr)
 		addEditor((*itr).path());
 }
+
+/**
+ * Ensures the "Show/Hide Query Window" action is unchecked when the dock
+ * is closed through its close button.
+ * This slot is conncted to the headerCloseButtonClicked() signal of the
+ * query window dock widget.
+ */
+void KScope::slotQueryDockClosed()
+{
+	m_pToggleQueryWindowAction->setChecked(false);
+}
+
+/**
+ * Ensures the "Show/Hide File View" action is unchecked when the dock
+ * is closed through its close button.
+ * This slot is conncted to the headerCloseButtonClicked() signal of the
+ * file view dock widget.
+ */
+void KScope::slotFileViewDockClosed()
+{
+	m_pToggleFileViewAction->setChecked(false);
+}
+
+#include "kscope.moc"
