@@ -1,18 +1,28 @@
 /***************************************************************************
-                          kscopeconfig.cpp  -  description
-                             -------------------
-    begin                : Fri Jun 13 2003
-    copyright            : (C) 2003 by Elad Lahav
-    email                : elad_lahav@users.sf.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
+ *
+ * Copyright (C) 2005 Elad Lahav (elad_lahav@users.sourceforge.net)
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
  ***************************************************************************/
 
 #include <kconfig.h>
@@ -20,32 +30,95 @@
 #include <kglobalsettings.h>
 #include "kscopeconfig.h"
 
+// NOTE:
+// This configuration file entry controls whether the welcome dialogue is
+// displayed. Normally it only needs to be shown once, but the entry's name
+// can be changed across versions to force the display of new information.
+#define SHOW_WELCOME_ENTRY "ShowWelcomeDlg"
+
+/**
+ * Stores the display name and the configuration file entry for a configurable
+ * GUI element.
+ * @author	Elad Lahav
+ */
+struct ElementInfo
+{
+	/** The display name of the element. */
+	const char* szName;
+	
+	/** The configuration file entry. */
+	const char* szEntry;
+};
+
+/**
+ * A list of GUI elements for which the colour can be configured.
+ */
+const ElementInfo eiColors[] = {
+	{ "File List (Foreground)", "FileListFore" },
+	{ "File List (Background)", "FileListBack" },
+	{ "Tag List (Foreground)", "TagListFore" },
+	{ "Tag List (Background)", "TagListBack" },
+	{ "Query Window (Foreground)", "QueryListFore" },
+	{ "Query Window (Background)", "QueryListBack" },
+	{ "Call Graph (Background)", "GraphBack" },
+	{ "Call Graph (Nodes)", "GraphNode" },
+	{ "Call Graph (Text)", "GraphText" },
+	{ "Call Graph (Multi-Call Nodes)", "GraphMultiCall" } 
+};
+
+/**
+ * A list of GUI elements for which the font can be configured.
+ */
+const ElementInfo eiFonts[] = {
+	{ "File List", "FileList" },
+	{ "Tag List", "TagList" },
+	{ "Query Page", "QueryList" },
+	{ "Call Graph", "Graph" }
+};
+
+#define COLOR_NAME(_i)	eiColors[_i].szName
+#define COLOR_ENTRY(_i)	eiColors[_i].szEntry
+#define FONT_NAME(_i)	eiFonts[_i].szName
+#define FONT_ENTRY(_i)	eiFonts[_i].szEntry
+
 KScopeConfig::ConfParams KScopeConfig::s_cpDef = {
-	"/usr/bin/cscope",
-	true,
-	"/usr/bin/ctags",
-	true,
-	SPLIT_SIZES(),
-	QColor(255, 255, 255),
-	QColor(0, 0, 0),
-	QColor(255, 255, 255),
-	QColor(0, 0, 0),
-	QColor(255, 255, 255),
-	QColor(0, 0, 0),
-	QFont(), // Font definitions are overriden in load()
-	QFont(),
-	QFont(),
-	100,
-	NameAsc,
-	false,
-	true,
-	true,
-	false,
-	true,
-	true,
+	"/usr/bin/cscope", // Cscope path
+	true, // Verbose flag for Cscope
+	"/usr/bin/ctags", // Ctags path
+	"/usr/bin/dot", // Dot path
+	true, // Show the tag list
+	SPLIT_SIZES(), // Tag list width
+	{
+		QColor(black), // File list foreground
+		QColor(white), // File list background
+		QColor(black), // Tag list foreground
+		QColor(white), // Tag list background
+		QColor(black), // Query page foreground
+		QColor(white), // Query page background
+		QColor("#c0c0c0"), // Call graph background
+		QColor("#c0ff80"), // Call graph nodes
+		QColor(black), // Call graph text
+		QColor("#ff8000")
+	},
+	{
+		QFont(), // Font definitions are overriden in load()
+		QFont(),
+		QFont(),
+		QFont()
+	},
+	NameAsc, // Ctags sort order
+	false, // Read-only mode
+	true, // Load last project
+	true, // Automatic tag highlighting
+	false, // Brief query captions
+	true, // Warn when file is modified on the disk
+	true, // Sort files when a project is loaded
 	"kate --line %L %F", // External editor example
-	Fast,
-	Embedded
+	Fast, // System profile
+	Embedded, // Editor context menu
+	"TB",  // Call graph orientation
+	10, // Maximum calls per graph node
+	0 // Default graph view
 };
 
 /**
@@ -67,13 +140,16 @@ KScopeConfig::~KScopeConfig()
  */
 void KScopeConfig::load()
 {
+	uint i;
+	
 	KConfig* pConf = kapp->config();
 
 	// Need a working instance to get the system's default font (cannot be
 	// initialised statically)
-	s_cpDef.fontFileList = KGlobalSettings::generalFont();
-	s_cpDef.fontTagList = KGlobalSettings::generalFont();
-	s_cpDef.fontQueryList = KGlobalSettings::generalFont();
+	s_cpDef.fonts[FileList] = KGlobalSettings::generalFont();
+	s_cpDef.fonts[TagList] = KGlobalSettings::generalFont();
+	s_cpDef.fonts[QueryWindow] = KGlobalSettings::generalFont();
+	s_cpDef.fonts[Graph] = KGlobalSettings::generalFont();
 	
 	// Read the paths to required executables
 	pConf->setGroup("Programs");
@@ -81,6 +157,7 @@ void KScopeConfig::load()
 	m_cp.bVerboseCscope = pConf->readBoolEntry("VerboseCScope",
 		s_cpDef.bVerboseCscope);
 	m_cp.sCtagsPath = pConf->readEntry("CTags");
+	m_cp.sDotPath = pConf->readEntry("Dot");
 
 	// Read size and position parameters
 	pConf->setGroup("Geometry");
@@ -89,8 +166,6 @@ void KScopeConfig::load()
 	m_cp.siEditor = pConf->readIntListEntry("Editor");
 	if (m_cp.siEditor.empty())
 		m_cp.siEditor << 200 << 800;
-	m_cp.bShowQueryButtons = pConf->readBoolEntry("ShowQueryButtons",
-		s_cpDef.bShowQueryButtons);
 
 	// Read the recent projects list
 	pConf->setGroup("Projects");
@@ -98,32 +173,20 @@ void KScopeConfig::load()
 
 	// Read colour settings
 	pConf->setGroup("Colors");
-	m_cp.clrFileListBack = pConf->readColorEntry("FileListBack",
-		&s_cpDef.clrFileListBack);
-	m_cp.clrFileListFore = pConf->readColorEntry("FileListFore",
-		&s_cpDef.clrFileListFore);
-	m_cp.clrTagListBack = pConf->readColorEntry("TagListBack",
-		&s_cpDef.clrTagListBack);
-	m_cp.clrTagListFore = pConf->readColorEntry("TagListFore",
-		&s_cpDef.clrTagListFore);
-	m_cp.clrQueryListBack = pConf->readColorEntry("QueryListBack",
-		&s_cpDef.clrQueryListBack);
-	m_cp.clrQueryListFore = pConf->readColorEntry("QueryListFore",
-		&s_cpDef.clrQueryListFore);
+	for (i = 0; i <= LAST_COLOR; i++) {
+		m_cp.clrs[i] = pConf->readColorEntry(COLOR_ENTRY(i),
+			&s_cpDef.clrs[i]);
+	}
 
 	// Read font settings
 	pConf->setGroup("Fonts");
-	m_cp.fontFileList = pConf->readFontEntry("FileList", 
-		&s_cpDef.fontFileList);
-	m_cp.fontTagList = pConf->readFontEntry("TagList",
-		&s_cpDef.fontTagList);
-	m_cp.fontQueryList = pConf->readFontEntry("QueryList",
-		&s_cpDef.fontQueryList);
+	for (i = 0; i <= LAST_FONT; i++) {
+		m_cp.fonts[i] = pConf->readFontEntry(FONT_ENTRY(i),
+			&s_cpDef.fonts[i]);
+	}
 	
 	// Other options
 	pConf->setGroup("Options");
-	m_cp.nMaxPosHistory = pConf->readUnsignedNumEntry("MaxPosHistory", 
-		s_cpDef.nMaxPosHistory);
 	m_cp.ctagSortOrder = 
 		(CtagSort)pConf->readUnsignedNumEntry("CtagSortOrder",
 		s_cpDef.ctagSortOrder);
@@ -137,11 +200,19 @@ void KScopeConfig::load()
 		s_cpDef.bBriefQueryCaptions);
 	m_cp.bWarnModifiedOnDisk = pConf->readBoolEntry("WarnModifiedOnDisk", 
 		s_cpDef.bWarnModifiedOnDisk);
+	m_cp.bAutoSortFiles = pConf->readBoolEntry("AutoSortFiles",
+		s_cpDef.bAutoSortFiles);
 	m_cp.sExtEditor = pConf->readEntry("ExternalEditor", s_cpDef.sExtEditor);
 	m_cp.profile = (SysProfile)pConf->readUnsignedNumEntry("SystemProfile",
 		s_cpDef.profile);
 	m_cp.popup = (EditorPopup)pConf->readUnsignedNumEntry("EditorPopup",
 		s_cpDef.popup);
+	m_cp.sGraphOrient = pConf->readEntry("GraphOrientation",
+		s_cpDef.sGraphOrient);
+	m_cp.nGraphMaxNodeDegree = pConf->readNumEntry("GraphMaxNodeDegree",
+		s_cpDef.nGraphMaxNodeDegree);
+	m_cp.nDefGraphView = pConf->readNumEntry("DefGraphView",
+		s_cpDef.nDefGraphView);
 }
 
 /**
@@ -167,6 +238,8 @@ void KScopeConfig::loadWorkspace(KDockMainWindow* pMainWindow)
  */
 void KScopeConfig::store()
 {
+	uint i;
+	
 	KConfig* pConf = kapp->config();
 
 	// Write the paths to required executables
@@ -174,12 +247,12 @@ void KScopeConfig::store()
 	pConf->writeEntry("CScope", m_cp.sCscopePath);
 	pConf->writeEntry("VerboseCScope", m_cp.bVerboseCscope);
 	pConf->writeEntry("CTags", m_cp.sCtagsPath);
+	pConf->writeEntry("Dot", m_cp.sDotPath);
 
 	// Write size and position parameters
 	pConf->setGroup("Geometry");
 	pConf->writeEntry("ShowTagList", m_cp.bShowTagList);
 	pConf->writeEntry("Editor", m_cp.siEditor);
-	pConf->writeEntry("ShowQueryButtons", m_cp.bShowQueryButtons);
 
 	// Write the recent projects list
 	pConf->setGroup("Projects");
@@ -187,19 +260,16 @@ void KScopeConfig::store()
 
 	// Write colour settings
 	pConf->setGroup("Colors");
-	pConf->writeEntry("FileListBack", m_cp.clrFileListBack);
-	pConf->writeEntry("FileListFore", m_cp.clrFileListFore);
-	pConf->writeEntry("TagListBack", m_cp.clrTagListBack);
-	pConf->writeEntry("TagListFore", m_cp.clrTagListFore);
-	pConf->writeEntry("QueryListBack", m_cp.clrQueryListBack);
-	pConf->writeEntry("QueryListFore", m_cp.clrQueryListFore);
+	for (i = 0; i <= LAST_COLOR; i++)
+		pConf->writeEntry(COLOR_ENTRY(i), m_cp.clrs[i]);
 
 	// Write font settings
 	if (m_bFontsChanged) {
 		pConf->setGroup("Fonts");
-		pConf->writeEntry("FileList", m_cp.fontFileList);
-		pConf->writeEntry("TagList", m_cp.fontTagList);
-		pConf->writeEntry("QueryList", m_cp.fontQueryList);
+		for (i = 0; i <= LAST_FONT; i++)
+			pConf->writeEntry(FONT_ENTRY(i), m_cp.fonts[i]);
+		
+		m_bFontsChanged = false;
 	}
 		
 	// Other options
@@ -210,13 +280,18 @@ void KScopeConfig::store()
 	pConf->writeEntry("AutoTagHl", m_cp.bAutoTagHl);
 	pConf->writeEntry("BriefQueryCaptions", m_cp.bBriefQueryCaptions);
 	pConf->writeEntry("WarnModifiedOnDisk", m_cp.bWarnModifiedOnDisk);
+	pConf->writeEntry("AutoSortFiles", m_cp.bAutoSortFiles);
 	pConf->writeEntry("ExternalEditor", m_cp.sExtEditor);
 	pConf->writeEntry("SystemProfile", (uint)m_cp.profile);
 	pConf->writeEntry("EditorPopup", (uint)m_cp.popup);
+	pConf->writeEntry("GraphOrientation", m_cp.sGraphOrient);
+	pConf->writeEntry("GraphMaxNodeDegree", m_cp.nGraphMaxNodeDegree);
+	pConf->writeEntry("DefGraphView", m_cp.nDefGraphView);
 	
 	// Do not report it's the first time on the next run
 	pConf->setGroup("General");
 	pConf->writeEntry("FirstTime", false);
+	pConf->writeEntry(SHOW_WELCOME_ENTRY, false);
 }
 
 /**
@@ -241,6 +316,21 @@ bool KScopeConfig::isFirstTime()
 	return pConf->readBoolEntry("FirstTime", true);
 }
 
+/**
+ * Determines if the welcome dialogue should be displayed.
+ * Note that while the dialogue is displayed on the first invocation of KScope,
+ * it may be required on other occasions (e.g., to display important information
+ * on a per-version basis) and thus it is separated from isFirstTime().
+ * @return	true if the dialogue should be shown, false otherwise
+ */
+bool KScopeConfig::showWelcomeDlg()
+{
+	KConfig* pConf = kapp->config();
+
+	pConf->setGroup("General");
+	return pConf->readBoolEntry(SHOW_WELCOME_ENTRY, true);
+}
+
 /**
  * @return	The full path of the Cscope executable
  */
@@ -283,13 +373,29 @@ const QString& KScopeConfig::getCtagsPath() const
 }
 
 /**
- * @param	sPath	The full path of the Cscope executable
+ * @param	sPath	The full path of the Ctags executable
  */
 void KScopeConfig::setCtagsPath(const QString& sPath)
 {
 	m_cp.sCtagsPath = sPath;
 }
 
+/**
+ * @return	The full path of the Dot executable
+ */
+const QString& KScopeConfig::getDotPath() const
+{
+	return m_cp.sDotPath;
+}
+
+/**
+ * @param	sPath	The full path of the Dot executable
+ */
+void KScopeConfig::setDotPath(const QString& sPath)
+{
+	m_cp.sDotPath = sPath;
+}
+
 /**
  * @return	A sorted list of recently used project paths.
  */
@@ -332,7 +438,7 @@ bool KScopeConfig::getShowTagList() const
 }
 
 /**
- * @param	bShowList	true to make the tag list visible, false otherwise
+ * @param	bShowTagList	true to make the tag list visible, false otherwise
  */
 void KScopeConfig::setShowTagList(bool bShowTagList)
 {
@@ -358,123 +464,64 @@ void KScopeConfig::setEditorSizes(const SPLIT_SIZES& siEditor)
 }
 
 /**
- * Finds a colour to use for either the foreground or the background of a
- * window. 
- * The windows for which KScope maintains colour preferences are the
- * children of the main widget (the query window and file list) and the tag
- * list. The colours of editor windows are managed by the editor part.
- * @param	cw		Identifies the window
- * @param	type	Whether the requested colour is for the foreground or the
- *					background of the window
+ * Finds a colour to use for a GUI element.
+ * @param	ce		Identifies the GUI element
  * @return	A reference to the colour object to use
  */
-const QColor& KScopeConfig::getColor(ConfWidget cw, ColorType type) const
+const QColor& KScopeConfig::getColor(ColorElement ce) const
 {
-	switch (cw) {
-	case FileList:
-		return (type == Background) ? m_cp.clrFileListBack :
-			m_cp.clrFileListFore;
-
-	case TagList:
-		return (type == Background) ? m_cp.clrTagListBack :
-			m_cp.clrTagListFore;
-
-	case QueryList:
-		return (type == Background) ? m_cp.clrQueryListBack :
-			m_cp.clrQueryListFore;
-	}
-
-	// Default return value, shouldn't happen (but else the compiler may
-	// complain)
-	return m_cp.clrFileListBack;
+	return m_cp.clrs[ce];
 }
 
 /**
- * Sets a new colour to a managed child window (@see getColor() for a list of
- * these windows.)
- * @param	cw		Identifies the window
- * @param	type	Whether the given colour is for the foreground or
- *					background of the window
+ * Returns the display name of a GUI element whose colour can be configured.
+ * @param	ce	The GUI element
+ * @return	A name used in the colour configuration page
+ */
+QString KScopeConfig::getColorName(ColorElement ce) const
+{
+	return COLOR_NAME(ce);
+}
+
+/**
+ * Sets a new colour to a GUI element.
+ * @param	ce		Identifies the GUI element
  * @param	clr		The colour to use
  */ 
-void KScopeConfig::setColor(ConfWidget cw, ColorType type, const QColor& clr)
+void KScopeConfig::setColor(ColorElement ce, const QColor& clr)
 {
-	switch (cw) {
-	case FileList:
-		(type == Background) ? m_cp.clrFileListBack = clr :
-			m_cp.clrFileListFore = clr;
-		break;
-		
-	case TagList:
-		(type == Background) ? m_cp.clrTagListBack = clr :
-			m_cp.clrTagListFore = clr;
-		break;
-
-	case QueryList:
-		(type == Background) ? m_cp.clrQueryListBack = clr :
-			m_cp.clrQueryListFore = clr;
-		break;
-	}
+	m_cp.clrs[ce] = clr;
 }
 
 /**
- * Finds a font to use for either the foreground or the background of a
- * window. 
- * The windows for which KScope maintains font preferences are the
- * children of the main widget (the query window and file list) and the tag
- * list. The font used by editor windows is managed by the editor part.
- * @param	cw		Identifies the window
+ * Finds a font to use for a GUI element.
+ * @param	fe		Identifies the GUI element
  * @return	A reference to the font object to use
  */
-const QFont& KScopeConfig::getFont(ConfWidget cw) const
+const QFont& KScopeConfig::getFont(FontElement fe) const
 {
-	switch (cw) {
-	case FileList:
-		return m_cp.fontFileList;
-		
-	case TagList:
-		return m_cp.fontTagList;
-		
-	case QueryList:
-		return m_cp.fontQueryList;
-	}
-
-	// Default return value, shouldn't happen (but else the compiler may
-	// complain)
-	return m_cp.fontFileList;
+	return m_cp.fonts[fe];
 }
 
 /**
- * Sets a new font to a managed child window (@see getFont() for a list of
- * these windows.)
- * @param	cw		Identifies the window
+ * Returns the display name of a GUI element whose font can be configured.
+ * @param	ce	The GUI element
+ * @return	A name used in the font configuration page
+ */
+QString KScopeConfig::getFontName(FontElement ce) const
+{
+	return FONT_NAME(ce);
+}
+
+/**
+ * Sets a new font to a GUI element.
+ * @param	fe		Identifies the GUI element
  * @param	font	The font to use
  */ 
-void KScopeConfig::setFont(ConfWidget cw, const QFont& font)
+void KScopeConfig::setFont(FontElement fe, const QFont& font)
 {
-	switch (cw) {
-	case FileList:
-		m_cp.fontFileList = font;
-		break;
-		
-	case TagList:
-		m_cp.fontTagList = font;
-		break;
-
-	case QueryList:
-		m_cp.fontQueryList = font;
-		break;
-	}
-
 	m_bFontsChanged = true;
-}
-
-/**
- * @return	The maximal size of the positions history list
- */
-uint KScopeConfig::getMaxPosHistory()
-{
-	return m_cp.nMaxPosHistory;
+	m_cp.fonts[fe] = font;
 }
 
 /**
@@ -563,24 +610,6 @@ void KScopeConfig::setUseBriefQueryCaptions(bool bBrief)
 	m_cp.bBriefQueryCaptions = bBrief;
 }
 
-/**
- * @return	true to show the toolbar to the side of the query window, false
- *			otherwise
- */
-bool KScopeConfig::getShowQueryButtons()
-{
-	return m_cp.bShowQueryButtons;
-}
-
-/**
- * @param	bShow	true to show the toolbar to the side of the query window,
- *					false otherwise
- */
-void KScopeConfig::setShowQueryButtons(bool bShow)
-{
-	m_cp.bShowQueryButtons = bShow;
-}
-
 /**
  * @return	true to warn user when file is modified on disk, false
  *			otherwise
@@ -599,6 +628,23 @@ void KScopeConfig::setWarnModifiedOnDisk(bool bWarn)
 	m_cp.bWarnModifiedOnDisk = bWarn;
 }
 
+/**
+ * @return	true to sort files when a project is loaded, false otherwise
+ */
+bool KScopeConfig::getAutoSortFiles()
+{
+	return m_cp.bAutoSortFiles;
+}
+
+/**
+ * @param	bWarn	true to sort files when a project is loaded, false 
+ *					otherwise
+ */
+void KScopeConfig::setAutoSortFiles(bool bSort)
+{
+	m_cp.bAutoSortFiles = bSort;
+}
+
 /**
  * @return	A command line for launching an external editor
  */
@@ -670,14 +716,62 @@ QString KScopeConfig::getEditorPopupName() const
 }
 
 /**
- * @param	ep	The popup menu to use for the embedded editor (@see
- *				EditorPopup)
+ * @param	popup	The popup menu to use for the embedded editor (@see
+ *					EditorPopup)
  */
 void KScopeConfig::setEditorPopup(KScopeConfig::EditorPopup popup)
 {
 	m_cp.popup = popup;
 }
 
+/**
+ * @return	The default orientation for call graphs
+ */
+QString KScopeConfig::getGraphOrientation() const
+{
+	return m_cp.sGraphOrient;
+}
+
+/**
+ * @param	sOrient	The default orientation for call graphs
+ */
+void KScopeConfig::setGraphOrientation(const QString& sOrient)
+{
+	m_cp.sGraphOrient = sOrient;
+}
+
+/**
+ * @return	The maximal number of calls per graph node
+ */
+int KScopeConfig::getGraphMaxNodeDegree() const
+{
+	return m_cp.nGraphMaxNodeDegree;
+}
+
+/**
+ * @param	nMaxNodeDegree	The maximal number of calls per graph node
+ */
+void KScopeConfig::setGraphMaxNodeDegree(int nMaxNodeDegree)
+{
+	m_cp.nGraphMaxNodeDegree = nMaxNodeDegree;
+}
+
+/**
+ * @return	The default view in the call graph dialogue
+ */
+int KScopeConfig::getDefGraphView() const
+{
+	return m_cp.nDefGraphView;
+}
+
+/**
+ * @param	nDefGraphView	The default view in the call graph dialogue
+ */
+void KScopeConfig::setDefGraphView(int nDefGraphView)
+{
+	m_cp.nDefGraphView = nDefGraphView;
+}
+
 /**
  * Returns a reference to a global configuration object.
  * The static object defined is this function should be the only KSCopeConfig
@@ -691,3 +785,5 @@ KScopeConfig& Config()
 	static KScopeConfig conf;
 	return conf;
 }
+
+#include "kscopeconfig.moc"
