@@ -1,18 +1,28 @@
 /***************************************************************************
-                          querywidget.cpp  -  description
-                             -------------------
-    begin                : Thu May 22 2003
-    copyright            : (C) 2003 by Elad Lahav
-    email                : elad_lahav@users.sf.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
+ *
+ * Copyright (C) 2005 Elad Lahav (elad_lahav@users.sourceforge.net)
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
  ***************************************************************************/
 
 #include <qtoolbutton.h>
@@ -21,7 +31,6 @@
 #include <kmessagebox.h>
 #include "querywidget.h"
 #include "kscopepixmaps.h"
-#include "cscopefrontend.h"
 #include "kscopeconfig.h"
 
 /**
@@ -31,79 +40,22 @@
  */
 QueryWidget::QueryWidget(QWidget* pParent, const char* szName) :
 	QueryWidgetLayout(pParent, szName),
+	m_pPageMenu(NULL),
+	m_pLockAction(NULL),
 	m_pHistPage(NULL),
 	m_bHistEnabled(true),
 	m_nQueryPages(0)
 {
-	QIconSet is;
-	
 	// Pages can be closed by clicking their tabs
 	m_pQueryTabs->setHoverCloseButton(true);
 	
-	// Initialise the "new query" button
-	m_pNewButton->setIconSet(
-		Pixmaps().getPixmap(KScopePixmaps::ButtonNewQuery));
-	QToolTip::add(m_pNewButton, i18n("New Query Page"));
-	
-	// Create a new query page when the "New Query" button is clicked
-	connect(m_pNewButton, SIGNAL(clicked()), this, 
-		SLOT(slotNewQueryPage()));
-	
-	// Create the icon set for the "lock query" button
-	is.setPixmap(Pixmaps().getPixmap(KScopePixmaps::ButtonLockQuery),
-		QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
-	is.setPixmap(Pixmaps().getPixmap(KScopePixmaps::ButtonUnlockQuery),
-		QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
-	
-	// Initialise the "lock query" toggle button
-	m_pLockButton->setIconSet(is);
-	QToolTip::add(m_pLockButton, i18n("Lock/Unlock Page"));
-	
-	// Lock/unlock a query when the toggle button changes state
-	connect(m_pLockButton, SIGNAL(toggled(bool)), this, 
-		SLOT(slotLockCurrent(bool)));
-	
-	// Lock/unlock a query when the toggle button changes state
-	connect(m_pQueryTabs, SIGNAL(currentChanged(QWidget*)), this, 
+	// Change the lock action state according to the current page
+	connect(m_pQueryTabs, SIGNAL(currentChanged(QWidget*)), this,
 		SLOT(slotCurrentChanged(QWidget*)));
 	
-	// Initialise the "refresh query" button
-	m_pRefreshButton->setIconSet(
-		Pixmaps().getPixmap(KScopePixmaps::ButtonRefreshQuery));
-	QToolTip::add(m_pRefreshButton, i18n("Refresh Query"));
-	
-	// Create a new query page when the "New Query" button is clicked
-	connect(m_pRefreshButton, SIGNAL(clicked()), this, 
-		SLOT(slotRefreshCurrent()));
-	
-	// Initialise the "close query button"
-	m_pCloseButton->setIconSet(
-		Pixmaps().getPixmap(KScopePixmaps::ButtonCloseQuery));
-	QToolTip::add(m_pCloseButton, i18n("Close Page"));
-	
-	// Close a query when the close button is clicked
-	connect(m_pCloseButton, SIGNAL(clicked()), this, 
-		SLOT(slotCloseCurrent()));
-	
-	// Close query when its tab button is clicked
+	// Close a query when its tab button is clicked
 	connect(m_pQueryTabs, SIGNAL(closeRequest(QWidget*)), this,
 		SLOT(slotClosePage(QWidget*)));
-	
-	// Hide the buttons, if not needed
-	if (!Config().getShowQueryButtons())
-		((QWidget*)m_pButtonGroup)->setShown(false);
-	
-	// Create a popup menu to control query page operations
-	m_pPageMenu = new QPopupMenu(this);
-	m_pPageMenu->insertItem(i18n("New Query Page"), this, 
-		SLOT(slotNewQueryPage()));
-	m_nLockMenuId = m_pPageMenu->insertItem(i18n("Lock/Unlock Page"), this, 
-		SLOT(slotLockCurrent()));
-	m_nRefreshMenuId = m_pPageMenu->insertItem(i18n("Refresh Query"), this, 
-		SLOT(slotRefreshCurrent()));
-	m_pPageMenu->insertSeparator();
-	m_nCloseMenuId = m_pPageMenu->insertItem(i18n("Close Page"), this, 
-		SLOT(slotCloseCurrent()));
 
 	// Show the menu when requested
 	connect(m_pQueryTabs, SIGNAL(contextMenu(const QPoint&)), this,
@@ -129,32 +81,23 @@ QueryWidget::~QueryWidget()
 void QueryWidget::initQuery(uint nType, const QString& sText)
 {
 	QueryPage* pPage;
-	CscopeFrontend* pCscope;
 	
 	// Make sure we have a query page
 	findQueryPage();
-	pPage = currentPage();
+	pPage = (QueryPage*)currentPage();
 			
 	// Use the current page, or a new page if the current one is locked
 	if (pPage->isLocked()) {
-		slotAddQueryPage();
-		pPage = currentPage();
+		addQueryPage();
+		pPage = (QueryPage*)currentPage();
 	}
 
 	// Reset the page's results list
 	pPage->clear();
-	pPage->setQueryData(nType, sText);
+	pPage->query(nType, sText);
 	
 	// Set the page's tab text according to the new query
 	setPageCaption(pPage);
-	
-	// Create a self-detsructing Cscope process, and set this page as the
-	// target for Cscope's output
-	pCscope = new CscopeFrontend(true);
-	pPage->attach(pCscope);
-	
-	// Run the query
-	pCscope->query(nType, sText);
 }
 
 /**
@@ -183,7 +126,7 @@ void QueryWidget::loadLockedQueries(const QString& sProjPath,
 	const QStringList& slQueryFiles)
 {
 	QStringList::ConstIterator itr;
-	QueryPage* pPage;
+	QueryPageBase* pPage;
 	QString sName;
 	
 	// Iterate through query files
@@ -195,7 +138,7 @@ void QueryWidget::loadLockedQueries(const QString& sProjPath,
 		}
 		else {
 			findQueryPage();
-			pPage = currentPage();
+			pPage = (QueryPage*)currentPage();
 		}
 
 		// Load a query file to this page, and lock the page
@@ -233,18 +176,6 @@ void QueryWidget::saveLockedQueries(const QString& sProjPath,
 	}
 }
 
-/**
- * Shows/hides the query buttons.
- */
-void QueryWidget::showHideButtons()
-{
-	bool bShow;
-	
-	bShow = !Config().getShowQueryButtons();
-	((QWidget*)m_pButtonGroup)->setShown(bShow);
-	Config().setShowQueryButtons(bShow);
-}
-
 /**
  * Adds a new position record to the active history page.
  * @param	sFile	The file path
@@ -274,7 +205,7 @@ void QueryWidget::addHistoryRecord(const QString& sFile, uint nLine,
  * Sets the tab caption and tool-tip for the given page.
  * @param	pPage	The page whose tab needs to be changed
  */
-void QueryWidget::setPageCaption(QueryPage* pPage)
+void QueryWidget::setPageCaption(QueryPageBase* pPage)
 {
 	m_pQueryTabs->changeTab(pPage, 
 		pPage->getCaption(Config().getUseBriefQueryCaptions()));
@@ -285,7 +216,7 @@ void QueryWidget::setPageCaption(QueryPage* pPage)
  * Creates a new query page, and adds it to the tab widget.
  * The new page is set as the current one.
  */
-void QueryWidget::slotAddQueryPage()
+void QueryWidget::addQueryPage()
 {
 	QueryPage* pPage;
 	QString sTitle;
@@ -294,19 +225,23 @@ void QueryWidget::slotAddQueryPage()
 	pPage = new QueryPage(this);
 
 	// Add the page, and set it as the current one
-	m_pQueryTabs->insertTab(pPage, 
-		Pixmaps().getPixmap(KScopePixmaps::TabUnlocked), "Query",
+	m_pQueryTabs->insertTab(pPage, GET_PIXMAP(TabUnlocked), "Query",
 		m_nQueryPages++);
 	setCurrentPage(pPage);
 	
 	// Emit the lineRequested() signal when a query record is selected on 
 	// this page
-	connect(pPage, SIGNAL(recordSelected(QListViewItem*)), this, 
-		SLOT(slotRequestLine(QListViewItem*)));
-	
-	// Emit the queryFinished() signal when all records have been added to
-	// the page
-	connect(pPage, SIGNAL(queryFinished()), this, SIGNAL(queryFinished()));
+	connect(pPage, SIGNAL(lineRequested(const QString&, uint)), this, 
+		SLOT(slotRequestLine(const QString&, uint)));
+}
+
+/**
+ * Creates a new query page, and emits signal about it.
+ */
+void QueryWidget::slotNewQueryPage()
+{
+	addQueryPage();
+	emit newQuery();
 }
 
 /**
@@ -317,7 +252,7 @@ void QueryWidget::slotAddQueryPage()
  */
 void QueryWidget::slotLockCurrent(bool bOn)
 {
-	QueryPage* pPage;
+	QueryPageBase* pPage;
 	
 	pPage = currentPage();
 	
@@ -330,7 +265,7 @@ void QueryWidget::slotLockCurrent(bool bOn)
  */
 void QueryWidget::slotLockCurrent()
 {
-	QueryPage* pPage;
+	QueryPageBase* pPage;
 	
 	pPage = currentPage();
 	if (pPage != NULL)
@@ -343,13 +278,12 @@ void QueryWidget::slotLockCurrent()
 void QueryWidget::slotRefreshCurrent()
 {
 	QueryPage* pPage;
-	CscopeFrontend* pCscope;
 	uint nType;
 	QString sText;
 	
 	// Make sure the current page is a valid, non-empty one
-	pPage = currentPage();
-	if ((pPage == NULL) || isHistoryPage(pPage) ||
+	pPage = dynamic_cast<QueryPage*>(currentPage());
+	if ((pPage == NULL) ||
 		(pPage->getQueryType() == CscopeFrontend::None)) {
 		return;
 	}
@@ -360,13 +294,33 @@ void QueryWidget::slotRefreshCurrent()
 	
 	// Clear the current page contents
 	pPage->clear();
-	pPage->setQueryData(nType, sText);
-	
-	// Create a self-destructing Cscope process, and run it with the page's
-	// parameters
-	pCscope = new CscopeFrontend(this);	
-	pPage->attach(pCscope);
-	pCscope->query(nType, sText);
+	pPage->query(nType, sText);
+}
+
+/**
+ * Selects the next query result record in the current query page.
+ */
+void QueryWidget::slotNextResult()
+{
+	QueryPage* pPage;
+
+	// Select the next record in the current page
+	pPage = dynamic_cast<QueryPage*>(currentPage());
+	if (pPage != NULL)
+		pPage->selectNext();
+}
+
+/**
+ * Selects the next query result record in the current query page.
+ */
+void QueryWidget::slotPrevResult()
+{
+	QueryPage* pPage;
+
+	// Select the next record in the current page
+	pPage = dynamic_cast<QueryPage*>(currentPage());
+	if (pPage != NULL)
+		pPage->selectPrev();
 }
 
 /**
@@ -412,14 +366,13 @@ void QueryWidget::slotAddHistoryPage()
 	m_pHistPage = new HistoryPage(this);
 
 	// Add the page, and set it as the current one
-	m_pQueryTabs->insertTab(m_pHistPage, 
-		Pixmaps().getPixmap(KScopePixmaps::TabUnlocked), "");
+	m_pQueryTabs->insertTab(m_pHistPage, GET_PIXMAP(TabUnlocked), "");
 	setPageCaption(m_pHistPage);
 
 	// Emit the lineRequested() signal when a query record is selected on 
 	// this page
-	connect(m_pHistPage, SIGNAL(recordSelected(QListViewItem*)), this, 
-		SLOT(slotRequestLine(QListViewItem*)));
+	connect(m_pHistPage, SIGNAL(lineRequested(const QString&, uint)), this, 
+		SLOT(slotRequestLine(const QString&, uint)));
 }
 
 /**
@@ -457,6 +410,19 @@ void QueryWidget::selectActiveHistory()
 		setCurrentPage(m_pHistPage);
 }
 
+/**
+ * Attaches the page operations menu to this widget.
+ * The page menu is a popup menu that handles such operations as opening a
+ * new page, closing a page, locking a page, etc.
+ * @param	pMenu	Pointer to the popup menu
+ * @param	pAction Pointer to the "Lock/Unlock" toggle action
+ */
+void QueryWidget::setPageMenu(QPopupMenu* pMenu, KToggleAction* pAction)
+{ 
+	m_pPageMenu = pMenu;
+	m_pLockAction = pAction;
+}
+
 /**
  * Emits a signal indicating a certain source file and line number are
  * requested.
@@ -465,24 +431,14 @@ void QueryWidget::selectActiveHistory()
  * editor page at the requested line.
  * @param	pItem	The list item selected by the user
  */
-void QueryWidget::slotRequestLine(QListViewItem* pItem)
+void QueryWidget::slotRequestLine(const QString& sFileName, uint nLine)
 {
-	QString sFileName, sLine;
-
-	// Get the file name and line number
-	sFileName = pItem->text(0);
-	sLine = pItem->text(2);
-
-	// Do not process the "No results" item
-	if (sLine.isEmpty())
-		return;
-	
 	// Disable history if the request came from the active history page
 	if (currentPage() == m_pHistPage)
 		m_bHistEnabled = false;
 		
 	// Emit the signal
-	emit lineRequested(sFileName, sLine.toUInt());
+	emit lineRequested(sFileName, nLine);
 	
 	// Re-enable history
 	if (currentPage() == m_pHistPage)
@@ -498,7 +454,7 @@ void QueryWidget::slotCurrentChanged(QWidget* pWidget)
 	QueryPage* pPage;
 	
 	pPage = (QueryPage*)pWidget;
-	m_pLockButton->setOn(pPage->isLocked());
+	m_pLockAction->setChecked(pPage->isLocked());
 }
 
 /**
@@ -532,15 +488,17 @@ void QueryWidget::slotClosePage(QWidget* pPage)
  * Displays a context menu for page operations.
  * This slot is connected to the contextMenu() signal, emitted by 
  * m_pQueryTabs.
+ * NOTE: We assume that the first item in the menu is "New".
  * @param	pt	The point over which the mouse was clicked in request for the
  *				context menu
  */
 void QueryWidget::slotContextMenu(const QPoint& pt)
 {
-	// Disable the lock and close operations (clicked outside any widget)
-	m_pPageMenu->setItemEnabled(m_nLockMenuId, false);
-	m_pPageMenu->setItemEnabled(m_nRefreshMenuId, false);
-	m_pPageMenu->setItemEnabled(m_nCloseMenuId, false);
+	uint i;
+	
+	// Disable everything but the "new" action (clicked outside any widget)
+	for (i = 1; i < m_pPageMenu->count(); i++)
+		m_pPageMenu->setItemEnabled(m_pPageMenu->idAt(i), false);
 	
 	// Show the menu
 	m_pPageMenu->popup(pt);
@@ -556,15 +514,15 @@ void QueryWidget::slotContextMenu(const QPoint& pt)
  */
 void QueryWidget::slotContextMenu(QWidget* pWidget, const QPoint& pt)
 {
+	uint i;
+	
 	// Operations are on the current page, so we must ensure the clicked
 	// tab becomes the current one
 	setCurrentPage(pWidget);
 	
-	// Enable the lock and close operations
-	m_pPageMenu->setItemEnabled(m_nLockMenuId, true);
-	m_pPageMenu->setItemEnabled(m_nRefreshMenuId,
-		dynamic_cast<HistoryPage*>(pWidget) == NULL);
-	m_pPageMenu->setItemEnabled(m_nCloseMenuId, true);
+	// Enable all operations
+	for (i = 1; i < m_pPageMenu->count(); i++)
+		m_pPageMenu->setItemEnabled(m_pPageMenu->idAt(i), true);
 	
 	// Show the menu
 	m_pPageMenu->popup(pt);
@@ -575,18 +533,16 @@ void QueryWidget::slotContextMenu(QWidget* pWidget, const QPoint& pt)
  * @param	pPage	The page to lock or unlock
  * @param	bLock	true to lock the page, false to unlock it
  */
-void QueryWidget::setPageLocked(QueryPage* pPage, bool bLock)
+void QueryWidget::setPageLocked(QueryPageBase* pPage, bool bLock)
 {
 	// Set the locking state of the current page
 	pPage->setLocked(bLock);
-	m_pQueryTabs->setTabIconSet(pPage,
-		bLock ? Pixmaps().getPixmap(KScopePixmaps::TabLocked) : 
-		Pixmaps().getPixmap(KScopePixmaps::TabUnlocked));
+	m_pQueryTabs->setTabIconSet(pPage, bLock ? GET_PIXMAP(TabLocked) : 
+		GET_PIXMAP(TabUnlocked));
 		
 	// There can only be one unlocked history page. Check if a non-active
 	// query page is being unlocked
-	if ((dynamic_cast<HistoryPage*>(pPage) != NULL) && (pPage != m_pHistPage)
-		&& (!bLock)) {
+	if (isHistoryPage(pPage) && (pPage != m_pHistPage)	&& !bLock) {
 		// Lock the active history page (may be NULL)
 		if (m_pHistPage != NULL)
 			setPageLocked(m_pHistPage, true);
@@ -607,23 +563,21 @@ void QueryWidget::setPageLocked(QueryPage* pPage, bool bLock)
 void QueryWidget::findQueryPage()
 {
 	QueryPage* pPage;
-	HistoryPage* pHistPage;
 	int nPages, i;
 	
 	// First check if the current page is an unlocked query page
-	pPage = currentPage();
-	if ((pPage != NULL) && 
-		(pHistPage = dynamic_cast<HistoryPage*>(pPage)) == NULL) {
-		if (!pPage->isLocked() && !pPage->isAttached())
+	pPage = dynamic_cast<QueryPage*>(currentPage());
+	if (pPage != NULL) {
+		if (!pPage->isLocked() && !pPage->isRunning())
 			return;
 	}
 	
 	// Look for the first unlocked query page
 	nPages = m_pQueryTabs->count();
 	for (i = 0; i < nPages; i++) {
-		pPage = (QueryPage*)m_pQueryTabs->page(i);
-		if ((pHistPage = dynamic_cast<HistoryPage*>(pPage)) == NULL) {
-			if (!pPage->isLocked() && !pPage->isAttached()) {
+		pPage = dynamic_cast<QueryPage*>(m_pQueryTabs->page(i));
+		if (pPage != NULL) {
+			if (!pPage->isLocked() && !pPage->isRunning()) {
 				setCurrentPage(pPage);
 				return;
 			}
@@ -631,7 +585,7 @@ void QueryWidget::findQueryPage()
 	}
 
 	// Couldn't find an unlocked query page, create a new one
-	slotAddQueryPage();
+	addQueryPage();
 }
 
 /**
@@ -641,7 +595,7 @@ void QueryWidget::findQueryPage()
  */
 void QueryWidget::findHistoryPage()
 {
-	QueryPage* pPage;
+	HistoryPage* pPage;
 	int nPages, i;
 	
 	// First check if the active history page is unlocked
@@ -651,10 +605,10 @@ void QueryWidget::findHistoryPage()
 	// Look for the first unlocked history page
 	nPages = m_pQueryTabs->count();
 	for (i = 0; i < nPages; i++) {
-		pPage = (QueryPage*)m_pQueryTabs->page(i);
-		if ((m_pHistPage = dynamic_cast<HistoryPage*>(pPage)) != NULL) {
-			if (!pPage->isLocked())
-				return;
+		pPage = dynamic_cast<HistoryPage*>(m_pQueryTabs->page(i));
+		if (pPage != NULL && !pPage->isLocked()) {
+			m_pHistPage = pPage;
+			return;
 		}
 	}
 
@@ -662,11 +616,4 @@ void QueryWidget::findHistoryPage()
 	slotAddHistoryPage();
 }
 
-/**
- * Creates a new query page, and emits signal about it.
- */
-void QueryWidget::slotNewQueryPage()
-{
-	slotAddQueryPage();
-	emit newQuery();
-}
+#include "querywidget.moc"
